
#include <SPI.h>
#include <Adafruit_VS1053.h>
#include <SD.h>
#include <MsTimer2.h>

#define LED_PIN 10

#define BREAKOUT_RESET  8      // VS1053 reset pin (output)
#define BREAKOUT_CS     6     // VS1053 chip select pin (output)
#define BREAKOUT_DCS    7      // VS1053 Data/command select pin (output)
#define CARDCS 9     // Card chip select pin
#define DREQ 2       // VS1053 Data request, ideally an Interrupt pin

Adafruit_VS1053_FilePlayer musicPlayer = 
  Adafruit_VS1053_FilePlayer(BREAKOUT_RESET, BREAKOUT_CS, BREAKOUT_DCS, DREQ, CARDCS);

int brightness = 0;    // how bright the LED is
int fadeAmount = 5;    // how many points to fade the LED by

boolean led_on = false;

String inputString = "";
String bufString = "";

void fade() {
  if( led_on ) {
    analogWrite(LED_PIN, brightness);
    brightness = brightness + fadeAmount;

    if (brightness <= 0 || brightness >= 255) {
      fadeAmount = -fadeAmount;
    }
  } else {
    brightness = 0;    // how bright the LED is
    fadeAmount = 5;    // how many points to fade the LED by
    analogWrite(LED_PIN, brightness);
  }
}

void setup() {
  Serial.begin(9600);
  Serial1.begin(19200);

  inputString.reserve(200);
  bufString.reserve(10);

  Serial.println("Adafruit VS1053 Simple Test");

  if (! musicPlayer.begin()) { // initialise the music player
     Serial.println(F("Couldn't find VS1053, do you have the right pins defined?"));
     while (1);
  }
  Serial.println(F("VS1053 found"));
  
   if (!SD.begin(CARDCS)) {
    Serial.println(F("SD failed, or not present"));
    while (1);  // don't do anything more
  }

  printDirectory(SD.open("/"), 0);
  musicPlayer.setVolume(20,20);
  musicPlayer.useInterrupt(VS1053_FILEPLAYER_PIN_INT);  // DREQ int
  
  Serial.println(F("Playing track 001"));

  pinMode(LED_PIN, OUTPUT);

  MsTimer2::set(50, fade); // 500ms period
  MsTimer2::start();
}

boolean detect = false;
unsigned char stat = 0;

void loop() {
  switch( stat ) {
    case 0:
      if ( Serial1.available() > 0 ) {
        char inChar = Serial1.read();
        inputString += inChar;
        if (inChar == '\n') {
          sensor_detect();
          inputString = "";
          bufString = "";
        }
        stat = 1;
      }
      break;
    case 1:
      if (detect) {
        if (musicPlayer.stopped()) {
          musicPlayer.startPlayingFile("/1.mp3");
          led_on = true;
          stat = 2;
        }
      } else {
        stat = 0;
      }
      break;
    case 2:
      if (musicPlayer.stopped()) {
          detect = false;
          led_on = false;
          int buf = Serial1.available(); 
          Serial.println( buf );
          for ( int i = 0 ; i < buf ; i++ ) {
            Serial.print( Serial1.read(), HEX ); // remove 1 character
            Serial.print( "," );
          }
          Serial.println("");
          stat = 0;
      }
      break;
  }
}


void printDirectory(File dir, int numTabs) {
   while(true) {
     File entry =  dir.openNextFile();
     if (! entry) {
       break;
     }
     for (uint8_t i=0; i<numTabs; i++) {
       Serial.print('\t');
     }
     Serial.print(entry.name());
     if (entry.isDirectory()) {
       Serial.println("/");
       printDirectory(entry, numTabs+1);
     } else {
       // files have sizes, directories do not
       Serial.print("\t\t");
       Serial.println(entry.size(), DEC);
     }
     entry.close();
   }
}

byte  reg;

void sensor_detect(void) {
  int dataindex;
  char **e;

  dataindex = inputString.indexOf(':') + 1;
  bufString = inputString.substring(dataindex, (dataindex + 2));
  const char *s = bufString.c_str();
  reg = (byte)strtol(s, e, 16);

  Serial.print( reg, HEX );
  Serial.println("");

  if( reg == 0x01 ) {
    detect = true;
  } else {
    detect = false;
  }
}
